'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

exports.default = initAppManager;

var _pathToRegexp = require('path-to-regexp');

var _pathToRegexp2 = _interopRequireDefault(_pathToRegexp);

var _decorateHistory = require('./utils/decorate-history');

var _decorateHistory2 = _interopRequireDefault(_decorateHistory);

var _getManagedSlotScripts = require('./utils/get-managed-slot-scripts');

var _getManagedSlotScripts2 = _interopRequireDefault(_getManagedSlotScripts);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DEFAULT_TIMEOUT = 4000;

var HISTORY_STATE_CHANGE = 'statechange';
var APP_MANAGER_ID = 'app-manager';

var appStatuses = {
  DEFAULT: 'DEFAULT',
  LOADING: 'LOADING',
  ERROR: 'ERROR'
};

function initAppManager(w, html5History) {
  var _class, _temp, _initialiseProps;

  var history = (0, _decorateHistory2.default)(w, html5History);

  return _temp = _class = function () {
    (0, _createClass3.default)(AppManager, null, [{
      key: '_getBrowserPath',
      value: function _getBrowserPath() {
        var browserState = history.getState();
        return (browserState.hash || '').split('?')[0];
      }
    }, {
      key: '_timeout',
      value: function _timeout() {
        var timeout = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_TIMEOUT;

        return new _promise2.default(function (resolve) {
          return setTimeout(resolve, timeout);
        });
      }
    }]);

    function AppManager(config, analytics, options) {
      (0, _classCallCheck3.default)(this, AppManager);

      _initialiseProps.call(this);

      if (!config) {
        throw new Error('Config not specified. Aborting.');
      }

      if (!w.document || !w.location) {
        throw new Error('Invalid window object. Aborting.');
      }

      var apps = config.apps,
          slots = config.slots,
          scripts = config.scripts;


      this._analytics = analytics || {
        error: function error(mock) {
          return mock;
        },
        namespace: 'mock' };
      this._apps = apps;
      this._slots = slots;
      this._scripts = scripts;
      this._options = options || {};
      this._currentAppName = null;
      this._initialised = false;
      this._status = appStatuses.DEFAULT;

      this._pathCheckers = (0, _keys2.default)(apps || {}).reduce(function (appPathCheckers, appName) {
        var appPath = apps[appName].appPath;


        if (!appPath || typeof appPath !== 'string') {
          return appPathCheckers;
        }

        return (0, _assign2.default)({}, appPathCheckers, (0, _defineProperty3.default)({}, appName, (0, _pathToRegexp2.default)(appPath)));
      }, {});

      var browserTitle = w.document.title;
      var browserPath = w.location.pathname;
      var browserQuery = w.location.search;

      history.replaceState({}, browserTitle, '' + browserPath + browserQuery);
      history.events.bind(HISTORY_STATE_CHANGE, APP_MANAGER_ID, this.onStateChange);
    }

    (0, _createClass3.default)(AppManager, [{
      key: '_goToPath',
      value: function _goToPath() {
        try {
          history.events.unbind(HISTORY_STATE_CHANGE, APP_MANAGER_ID);

          var newPath = this.constructor._getBrowserPath();
          w.location.href = newPath; // eslint-disable-line no-param-reassign
          return true;
        } catch (err) {
          this._analytics.error({ eventTitle: this._analytics.namespace + '.go_to_path.update_href' });
          return false;
        }
      }
    }]);
    return AppManager;
  }(), _initialiseProps = function _initialiseProps() {
    var _this = this;

    this._setStatus = function (newStatus) {
      if (_this._status !== newStatus) {
        _this._status = newStatus;
        history.events.trigger('statuschange', newStatus);
      }
    };

    this._getAppFromPath = function () {
      var path = _this.constructor._getBrowserPath();

      if (!path || typeof path !== 'string') {
        _this._analytics.error({ eventTitle: _this._analytics.namespace + '.get_app_from_path.no_path' });
        return null;
      }

      return (0, _keys2.default)(_this._pathCheckers).find(function (appName) {
        var pathChecker = _this._pathCheckers[appName];

        if (!pathChecker) {
          _this._analytics.error({ eventTitle: _this._analytics.namespace + '.get_app_from_path.invalid_appPath', id: appName });
          return false;
        }

        return pathChecker.exec(path);
      });
    };

    this._getSlotElement = function (slotName) {
      var slot = _this._slots[slotName];

      if (!slot) {
        _this._analytics.error({ eventTitle: _this._analytics.namespace + '.get_element.no_slot', id: slotName });
        return null;
      }

      var element = w.document.querySelector('.' + slot.elementClass);

      if (!element) {
        _this._analytics.error({ eventTitle: _this._analytics.namespace + '.get_element.no_element', id: slotName });
        return null;
      }

      return element;
    };

    this._getAppScript = function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(appScriptName) {
        var fragmentScript, newScript;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (!(_this._appScripts && _this._appScripts[appScriptName])) {
                  _context2.next = 2;
                  break;
                }

                return _context2.abrupt('return', _this._appScripts[appScriptName]());

              case 2:
                fragmentScript = _this._scripts[appScriptName];

                if (fragmentScript) {
                  _context2.next = 6;
                  break;
                }

                _this._analytics.error({ eventTitle: _this._analytics.namespace + '.get_script.missing', id: appScriptName });
                return _context2.abrupt('return', null);

              case 6:
                if (!(typeof fragmentScript.load !== 'function')) {
                  _context2.next = 9;
                  break;
                }

                _this._analytics.error({ eventTitle: _this._analytics.namespace + '.get_script.not_function', id: appScriptName });
                return _context2.abrupt('return', null);

              case 9:
                _context2.prev = 9;
                _context2.next = 12;
                return _promise2.default.race([fragmentScript.load(), _this.constructor._timeout(_this._options.importTimeout)]);

              case 12:
                newScript = _context2.sent;

                if (newScript) {
                  _context2.next = 16;
                  break;
                }

                _this._analytics.error({ eventTitle: _this._analytics.namespace + '.get_script.no_file', id: appScriptName });
                return _context2.abrupt('return', null);

              case 16:
                if ('version' in newScript) {
                  _context2.next = 19;
                  break;
                }

                _this._analytics.error({ eventTitle: _this._analytics.namespace + '.get_script.invalid_script', id: appScriptName });
                return _context2.abrupt('return', null);

              case 19:

                _this._appScripts = (0, _assign2.default)({}, _this._appScripts, (0, _defineProperty3.default)({}, appScriptName, (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
                  return _regenerator2.default.wrap(function _callee$(_context) {
                    while (1) {
                      switch (_context.prev = _context.next) {
                        case 0:
                          return _context.abrupt('return', newScript);

                        case 1:
                        case 'end':
                          return _context.stop();
                      }
                    }
                  }, _callee, _this);
                }))));

                return _context2.abrupt('return', newScript);

              case 23:
                _context2.prev = 23;
                _context2.t0 = _context2['catch'](9);

                _this._analytics.error({ eventTitle: _this._analytics.namespace + '.get_script.load', id: appScriptName });
                return _context2.abrupt('return', null);

              case 27:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, _this, [[9, 23]]);
      }));

      return function (_x2) {
        return _ref.apply(this, arguments);
      };
    }();

    this._getPageScript = function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(slotScripts, slotName) {
        var appScript;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return _this._getAppScript(slotScripts[slotName]);

              case 2:
                appScript = _context3.sent;
                return _context3.abrupt('return', appScript ? { script: appScript, slot: slotName, name: slotScripts[slotName] } : null);

              case 4:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, _this);
      }));

      return function (_x3, _x4) {
        return _ref3.apply(this, arguments);
      };
    }();

    this._getPageScripts = function () {
      var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(appName) {
        var slotScripts, appScriptImportPromises;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                slotScripts = (0, _getManagedSlotScripts2.default)(_this._slots, _this._scripts, _this._apps[appName]);

                if (slotScripts) {
                  _context4.next = 4;
                  break;
                }

                _this._analytics.error({ eventTitle: _this._analytics.namespace + '.get_page_scripts.get', id: appName });
                return _context4.abrupt('return', null);

              case 4:
                appScriptImportPromises = (0, _keys2.default)(slotScripts).reduce(function (pageScripts, slotName) {
                  if (!slotScripts[slotName]) {
                    return pageScripts;
                  }

                  return pageScripts.concat(_this._getPageScript(slotScripts, slotName));
                }, []);
                return _context4.abrupt('return', _promise2.default.all(appScriptImportPromises));

              case 6:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, _this);
      }));

      return function (_x5) {
        return _ref4.apply(this, arguments);
      };
    }();

    this._hydrateScript = function (appName) {
      return function () {
        var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(appScript) {
          var slotElement, newApp, script;
          return _regenerator2.default.wrap(function _callee5$(_context5) {
            while (1) {
              switch (_context5.prev = _context5.next) {
                case 0:
                  slotElement = _this._getSlotElement(appScript.slot);

                  if (slotElement) {
                    _context5.next = 4;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.hydrate.missing_element.' + appScript.slot, id: appScript.name || appName });
                  return _context5.abrupt('return', false);

                case 4:
                  newApp = _this._apps[appName];

                  if (newApp) {
                    _context5.next = 8;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.hydrate.invalid_app', id: appName });
                  return _context5.abrupt('return', false);

                case 8:
                  if (appScript.script) {
                    _context5.next = 11;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.hydrate.missing_script', id: appName });
                  return _context5.abrupt('return', false);

                case 11:
                  if (!(typeof appScript.script.hydrate !== 'function')) {
                    _context5.next = 14;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.hydrate.invalid_script', id: appScript.name || appName });
                  return _context5.abrupt('return', false);

                case 14:
                  script = appScript.script;
                  _context5.prev = 15;
                  _context5.next = 18;
                  return script.hydrate(slotElement, history, newApp);

                case 18:
                  return _context5.abrupt('return', true);

                case 21:
                  _context5.prev = 21;
                  _context5.t0 = _context5['catch'](15);

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.hydrate.run_script', id: appScript.name });
                  return _context5.abrupt('return', false);

                case 25:
                case 'end':
                  return _context5.stop();
              }
            }
          }, _callee5, _this, [[15, 21]]);
        }));

        return function (_x6) {
          return _ref5.apply(this, arguments);
        };
      }();
    };

    this._unmountScript = function (appName) {
      return function () {
        var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(appScript) {
          var slotElement, newApp, script;
          return _regenerator2.default.wrap(function _callee6$(_context6) {
            while (1) {
              switch (_context6.prev = _context6.next) {
                case 0:
                  slotElement = _this._getSlotElement(appScript.slot);

                  if (slotElement) {
                    _context6.next = 4;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.unmount.missing', id: appScript.name });
                  return _context6.abrupt('return', false);

                case 4:
                  newApp = _this._apps[appName];

                  if (newApp) {
                    _context6.next = 8;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.unmount.invalid_app', id: appName });
                  return _context6.abrupt('return', false);

                case 8:
                  if (appScript.script) {
                    _context6.next = 11;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.unmount.missing_script', id: appName });
                  return _context6.abrupt('return', false);

                case 11:
                  if (!(typeof appScript.script.unmount !== 'function')) {
                    _context6.next = 14;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.unmount.invalid_script', id: appScript.name || appName });
                  return _context6.abrupt('return', false);

                case 14:
                  script = appScript.script;
                  _context6.prev = 15;
                  _context6.next = 18;
                  return script.unmount(slotElement, history, newApp);

                case 18:
                  return _context6.abrupt('return', true);

                case 21:
                  _context6.prev = 21;
                  _context6.t0 = _context6['catch'](15);

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.unmount.error', id: appScript.name || appName });
                  return _context6.abrupt('return', false);

                case 25:
                case 'end':
                  return _context6.stop();
              }
            }
          }, _callee6, _this, [[15, 21]]);
        }));

        return function (_x7) {
          return _ref6.apply(this, arguments);
        };
      }();
    };

    this._mountScript = function (appName) {
      return function () {
        var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(appScript) {
          var slotElement, newApp, script;
          return _regenerator2.default.wrap(function _callee7$(_context7) {
            while (1) {
              switch (_context7.prev = _context7.next) {
                case 0:
                  slotElement = _this._getSlotElement(appScript.slot);

                  if (slotElement) {
                    _context7.next = 4;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.mount.missing', id: appScript.name || appName });
                  return _context7.abrupt('return', false);

                case 4:
                  newApp = _this._apps[appName];

                  if (newApp) {
                    _context7.next = 8;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.mount.invalid_app', id: appName });
                  return _context7.abrupt('return', false);

                case 8:
                  if (appScript.script) {
                    _context7.next = 11;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.mount.missing_script', id: appName });
                  return _context7.abrupt('return', false);

                case 11:
                  if (!(typeof appScript.script.mount !== 'function')) {
                    _context7.next = 14;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.mount.invalid_script', id: appScript.name || appName });
                  return _context7.abrupt('return', false);

                case 14:
                  script = appScript.script;
                  _context7.prev = 15;
                  _context7.next = 18;
                  return script.mount(slotElement, history, newApp);

                case 18:
                  return _context7.abrupt('return', true);

                case 21:
                  _context7.prev = 21;
                  _context7.t0 = _context7['catch'](15);

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.mount.error', id: appScript.name || appName });
                  return _context7.abrupt('return', false);

                case 25:
                case 'end':
                  return _context7.stop();
              }
            }
          }, _callee7, _this, [[15, 21]]);
        }));

        return function (_x8) {
          return _ref7.apply(this, arguments);
        };
      }();
    };

    this._stateChangeScript = function (appName) {
      return function () {
        var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(appScript) {
          var updateApp, script;
          return _regenerator2.default.wrap(function _callee8$(_context8) {
            while (1) {
              switch (_context8.prev = _context8.next) {
                case 0:
                  updateApp = _this._apps[appName];

                  if (updateApp) {
                    _context8.next = 4;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.state_change.invalid_app', id: appName });
                  return _context8.abrupt('return', false);

                case 4:
                  if (appScript.script) {
                    _context8.next = 7;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.state_change.missing_script', id: appName });
                  return _context8.abrupt('return', false);

                case 7:
                  if (!(typeof appScript.script.onStateChange !== 'function')) {
                    _context8.next = 10;
                    break;
                  }

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.state_change.invalid_script', id: appScript.name || appName });
                  return _context8.abrupt('return', false);

                case 10:
                  script = appScript.script;
                  _context8.prev = 11;
                  _context8.next = 14;
                  return script.onStateChange(history, updateApp);

                case 14:
                  return _context8.abrupt('return', true);

                case 17:
                  _context8.prev = 17;
                  _context8.t0 = _context8['catch'](11);

                  _this._analytics.error({ eventTitle: _this._analytics.namespace + '.state_change.error', id: appScript.name || appName });
                  return _context8.abrupt('return', false);

                case 21:
                case 'end':
                  return _context8.stop();
              }
            }
          }, _callee8, _this, [[11, 17]]);
        }));

        return function (_x9) {
          return _ref8.apply(this, arguments);
        };
      }();
    };

    this._hydratePage = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9() {
      var currentAppName, pageScripts, scripts, hydratePromises, hydrateSuccess;
      return _regenerator2.default.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              currentAppName = _this._currentAppName;

              if (currentAppName) {
                _context9.next = 4;
                break;
              }

              _this._analytics.error({ eventTitle: _this._analytics.namespace + '.hydrate_page.missing_route' });
              return _context9.abrupt('return', false);

            case 4:
              _context9.prev = 4;
              _context9.next = 7;
              return _this._getPageScripts(currentAppName);

            case 7:
              pageScripts = _context9.sent;

              if (!(!Array.isArray(pageScripts) || !pageScripts.every(Boolean))) {
                _context9.next = 10;
                break;
              }

              return _context9.abrupt('return', false);

            case 10:
              scripts = pageScripts;
              hydratePromises = scripts.map(_this._hydrateScript(currentAppName));
              _context9.next = 14;
              return _promise2.default.all(hydratePromises);

            case 14:
              hydrateSuccess = _context9.sent;
              return _context9.abrupt('return', hydrateSuccess.every(Boolean));

            case 18:
              _context9.prev = 18;
              _context9.t0 = _context9['catch'](4);

              _this._analytics.error({ eventTitle: _this._analytics.namespace + '.hydrate_page.get_scripts', id: currentAppName });
              return _context9.abrupt('return', false);

            case 22:
            case 'end':
              return _context9.stop();
          }
        }
      }, _callee9, _this, [[4, 18]]);
    }));

    this._updatePage = function () {
      var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10(newAppName) {
        var oldAppName, _ref11, _ref12, oldAppScripts, newAppScripts, oldScripts, newScripts, oldAppScriptNames, newAppScriptNames, unmountPromises, mountPromises, updatePromises, unmountSuccess, mountSuccess, updateSuccess;

        return _regenerator2.default.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                oldAppName = _this._currentAppName;

                if (oldAppName) {
                  _context10.next = 4;
                  break;
                }

                _this._analytics.error({ eventTitle: _this._analytics.namespace + '.update_page.no_current_app' });
                return _context10.abrupt('return', false);

              case 4:
                _context10.next = 6;
                return _promise2.default.all([_this._getPageScripts(oldAppName), _this._getPageScripts(newAppName)]);

              case 6:
                _ref11 = _context10.sent;
                _ref12 = (0, _slicedToArray3.default)(_ref11, 2);
                oldAppScripts = _ref12[0];
                newAppScripts = _ref12[1];

                if (!(!Array.isArray(oldAppScripts) || !Array.isArray(newAppScripts) || !oldAppScripts.every(Boolean) || !newAppScripts.every(Boolean))) {
                  _context10.next = 12;
                  break;
                }

                return _context10.abrupt('return', false);

              case 12:
                oldScripts = oldAppScripts;
                newScripts = newAppScripts;
                oldAppScriptNames = oldScripts.map(function (_ref13) {
                  var name = _ref13.name;
                  return name;
                });
                newAppScriptNames = newScripts.map(function (_ref14) {
                  var name = _ref14.name;
                  return name;
                });
                unmountPromises = oldScripts.filter(function (appScript) {
                  return newAppScriptNames.indexOf(appScript.name) === -1;
                }).map(_this._unmountScript(oldAppName));
                mountPromises = newScripts.filter(function (appScript) {
                  return oldAppScriptNames.indexOf(appScript.name) === -1;
                }).map(_this._mountScript(newAppName));
                updatePromises = oldScripts.filter(function (appScript) {
                  return newAppScriptNames.indexOf(appScript.name) > -1;
                }).map(_this._stateChangeScript(newAppName));
                _context10.next = 21;
                return _promise2.default.all(unmountPromises);

              case 21:
                unmountSuccess = _context10.sent;

                if (unmountSuccess.every(Boolean)) {
                  _context10.next = 24;
                  break;
                }

                return _context10.abrupt('return', false);

              case 24:
                _context10.next = 26;
                return _promise2.default.all(mountPromises);

              case 26:
                mountSuccess = _context10.sent;

                if (mountSuccess.every(Boolean)) {
                  _context10.next = 29;
                  break;
                }

                return _context10.abrupt('return', false);

              case 29:
                _context10.next = 31;
                return _promise2.default.all(updatePromises);

              case 31:
                updateSuccess = _context10.sent;

                if (updateSuccess.every(Boolean)) {
                  _context10.next = 34;
                  break;
                }

                return _context10.abrupt('return', false);

              case 34:
                return _context10.abrupt('return', true);

              case 35:
              case 'end':
                return _context10.stop();
            }
          }
        }, _callee10, _this);
      }));

      return function (_x10) {
        return _ref10.apply(this, arguments);
      };
    }();

    this.onStateChange = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11() {
      var newAppName, updatePageSuccess;
      return _regenerator2.default.wrap(function _callee11$(_context11) {
        while (1) {
          switch (_context11.prev = _context11.next) {
            case 0:
              if (_this._initialised) {
                _context11.next = 2;
                break;
              }

              return _context11.abrupt('return', false);

            case 2:
              if (!(_this._status === appStatuses.ERROR)) {
                _context11.next = 5;
                break;
              }

              _this._goToPath();
              return _context11.abrupt('return', false);

            case 5:

              _this._setStatus(appStatuses.LOADING);

              newAppName = _this._getAppFromPath();

              if (newAppName) {
                _context11.next = 12;
                break;
              }

              _this._analytics.error({ eventTitle: _this._analytics.namespace + '.update_app.app_not_found' });
              _this._setStatus(appStatuses.ERROR);
              _this._goToPath();
              return _context11.abrupt('return', false);

            case 12:
              _context11.prev = 12;
              _context11.next = 15;
              return _this._updatePage(newAppName);

            case 15:
              updatePageSuccess = _context11.sent;

              if (updatePageSuccess) {
                _context11.next = 20;
                break;
              }

              _this._setStatus(appStatuses.ERROR);
              _this._goToPath();
              return _context11.abrupt('return', false);

            case 20:

              _this._currentAppName = newAppName;
              _this._setStatus(appStatuses.DEFAULT);

              return _context11.abrupt('return', true);

            case 25:
              _context11.prev = 25;
              _context11.t0 = _context11['catch'](12);

              _this._analytics.error({ eventTitle: _this._analytics.namespace + '.update_app', id: newAppName });
              _this._setStatus(appStatuses.ERROR);
              _this._goToPath();
              return _context11.abrupt('return', false);

            case 31:
            case 'end':
              return _context11.stop();
          }
        }
      }, _callee11, _this, [[12, 25]]);
    }));

    this._initError = function (type) {
      if (type) {
        _this._analytics.error({ eventTitle: _this._analytics.namespace + '.init.' + type });
      }

      _this._setStatus(appStatuses.ERROR);
      history.events.unbind(HISTORY_STATE_CHANGE, APP_MANAGER_ID);
      return false;
    };

    this.init = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee12() {
      var currentAppName, hydrateSuccess;
      return _regenerator2.default.wrap(function _callee12$(_context12) {
        while (1) {
          switch (_context12.prev = _context12.next) {
            case 0:
              _this._setStatus(appStatuses.LOADING);

              if ((0, _keys2.default)(_this._slots).every(function (slotName) {
                return _this._getSlotElement(slotName);
              })) {
                _context12.next = 3;
                break;
              }

              return _context12.abrupt('return', _this._initError('missing_elements'));

            case 3:
              currentAppName = _this._getAppFromPath();

              if (currentAppName) {
                _context12.next = 6;
                break;
              }

              return _context12.abrupt('return', _this._initError('no_route'));

            case 6:

              _this._currentAppName = currentAppName;

              _context12.next = 9;
              return _this._hydratePage();

            case 9:
              hydrateSuccess = _context12.sent;

              if (hydrateSuccess) {
                _context12.next = 12;
                break;
              }

              return _context12.abrupt('return', _this._initError());

            case 12:

              _this._initialised = true;
              _this._setStatus(appStatuses.DEFAULT);

              return _context12.abrupt('return', true);

            case 15:
            case 'end':
              return _context12.stop();
          }
        }
      }, _callee12, _this);
    }));
  }, _temp;
}