'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

exports.default = getManagedSlotScripts;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getManagedScripts() {
  var slots = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var scripts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var appScripts = arguments[2];

  var availableSlots = (0, _keys2.default)(slots);

  return appScripts.map(function (scriptName) {
    return scripts[scriptName];
  }).filter(function (script) {
    return script && Array.isArray(script.slots) && script.managed === true;
  }).map(function (script) {
    return (0, _assign2.default)({}, script, { slots: script.slots.filter(function (slotName) {
        return availableSlots.includes(slotName);
      }) });
  }).filter(function (script) {
    return script.slots.length > 0;
  });
}

function filterLessImportantScriptSlots(scripts, neededSlot, startIndex) {
  function filterScriptSlots(filteredScripts, script, index) {
    if (index <= startIndex) {
      return filteredScripts;
    }

    return [].concat((0, _toConsumableArray3.default)(filteredScripts.slice(0, index)), [(0, _assign2.default)({}, script, { slots: script.slots.filter(function (slotName) {
        return slotName !== neededSlot;
      }) })], (0, _toConsumableArray3.default)(filteredScripts.slice(index + 1)));
  }

  return scripts.reduce(filterScriptSlots, scripts);
}

function ensurePriorityScripts(scripts) {
  function filterScriptSlots(filteredScripts, script, index) {
    return script.slots.length === 1 ? filterLessImportantScriptSlots(filteredScripts, script.slots[0], index) : filteredScripts;
  }

  return scripts.reduce(filterScriptSlots, scripts);
}

function getChain(scripts) {
  var usedSlots = [];

  function findFirstSeen(script) {
    var slot = script.slots[0];

    if (usedSlots.includes(slot)) {
      return true;
    }

    usedSlots.push(slot);
    return false;
  }

  var firstSeenIndex = scripts.findIndex(findFirstSeen);
  return firstSeenIndex === -1 ? scripts : scripts.slice(0, firstSeenIndex);
}

function calculateSlots(allScripts, numSlots) {
  function getSlots(test, offset, scripts, longestChain) {
    function findLongestChainInScripts(scriptChain, script, scriptIndex) {
      if (scriptChain.length >= numSlots) {
        return scriptChain;
      }

      var lessImportantScripts = scripts.slice(scriptIndex + 1);

      function findLongestChainInSlots(slotChain, slot) {
        if (slotChain.length >= numSlots) {
          return slotChain;
        }

        var nextTest = [].concat((0, _toConsumableArray3.default)(test.slice(0, scriptIndex + offset)), [(0, _assign2.default)({}, script, { slots: [slot] })], (0, _toConsumableArray3.default)(test.slice(scriptIndex + offset + 1)));

        if (lessImportantScripts.length > 0) {
          return getSlots(nextTest, offset + 1, lessImportantScripts, slotChain);
        }

        var nextChain = getChain(nextTest);
        return nextChain.length > slotChain.length ? nextChain : slotChain;
      }

      return script.slots.reduce(findLongestChainInSlots, scriptChain);
    }

    return scripts.reduce(findLongestChainInScripts, longestChain);
  }

  var seed = allScripts.map(function (script) {
    return (0, _assign2.default)({}, script, { slots: [script.slots[0]] });
  });
  return getSlots(seed, 0, allScripts, []);
}

function getManagedSlotScripts() {
  var slots = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var scripts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var app = arguments[2];

  if (!app || !Array.isArray(app.scripts)) {
    return null;
  }

  var managedScripts = getManagedScripts(slots, scripts, app.scripts);
  var availableSlots = (0, _keys2.default)(slots);
  var ensuredScripts = [].concat((0, _toConsumableArray3.default)(Array(availableSlots.length))).reduce(ensurePriorityScripts, managedScripts);
  var filteredScripts = calculateSlots(ensuredScripts, availableSlots.length);
  var emptyAppSlots = availableSlots.reduce(function (emptySlots, slotName) {
    return (0, _assign2.default)(emptySlots, (0, _defineProperty3.default)({}, slots[slotName].name, null));
  }, {});
  return filteredScripts.reduce(function (scriptsInSlots, script) {
    return (0, _assign2.default)({}, scriptsInSlots, (0, _defineProperty3.default)({}, script.slots[0], script.name));
  }, emptyAppSlots);
}